package com.diagbot.repository;

import com.diagbot.entity.node.BillConcept;
import com.diagbot.entity.node.BillCondition;
import com.diagbot.entity.node.BillItem;
import com.diagbot.entity.node.BillItemDetail;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/11/17 13:26
 */
public interface BillManRepository extends Neo4jRepository<BillItemDetail,Long> {

    @Query(value = "match(m)-[r]->(n) \n" +
            "where 1 = 1 \n" +
            "and (m:实验室检查套餐名 or m:辅助检查名称 or m:辅助检查子项目名称 or m:医保手术和操作名称 or m:药品注册名称) \n" +
            "and (n:性别 or n:实验室检查 or n:临床表现 or n:辅助检查名称描述 or n:疾病 or n:服用药品 or n:药物过敏原 or n:食物过敏原 or n:手术 or n:禁忌人群 or n:过敏原 or n:体征 or n:禁忌医疗器械及物品 or n:年龄 or n:给药途径 or n:开单项互斥) \n" +
            //"and (case {labels} is not null and size({labels})>0 when true then labels(m)[0] in {labels} else 1 = 1 end ) \n" +
            "and (case {conceptName} is not null and {conceptName} <> '' when true then m.name contains {conceptName} else 1 = 1 end ) \n" +
            "and (case {conditionName} is not null and {conditionName} <> '' when true then " +
            "(case  \n" +
            "when exists(n.禁忌人群指标) then toLower(n.禁忌人群指标) contains toLower({conditionName}) \n" +
            "when exists(n.实验室检查指标) then toLower(n.实验室检查指标) contains toLower({conditionName}) \n" +
            "when exists(n.体征指标) then toLower(n.体征指标) contains toLower({conditionName}) \n" +
            "else toLower(n.`name`) contains toLower({conditionName}) \n" +
            "end ) \n" +
            "else 1 = 1 " +
            "end ) \n" +
            "and (case {relationStatus} is not null  when true then r.状态 = {relationStatus} else 1 = 1 end ) \n" +
            "and type(r) contains('禁忌') \n" +
            "and m.状态=1 \n" +
            "and n.状态=1 \n" +
            "return " +
            "r.规则类型 as ruleType,\n" +
            "case labels(m)[0] " +
            "when '实验室检查套餐名' then 1 " +
            "when '辅助检查名称' then 2 " +
            "when '辅助检查子项目名称' then 3 " +
            "when '医保手术和操作名称' then 4 " +
            "when '药品注册名称' then 5 " +
            "end as type," +
            "id(m) as conceptId,m.name as conceptName,labels(m)[0] as conceptType,\n" +
            "id(r) as relationId,type(r) as relationName,r.状态 as relationStatus, \n" +
            "id(n) as conditionId," +
            //"n.name as conditionName," +
            "(case \n" +
            "when exists(n.禁忌人群指标) then n.禁忌人群指标 \n" +
            "when exists(n.实验室检查指标) then n.实验室检查指标 \n" +
            "when exists(n.体征指标) then n.体征指标 \n" +
            "else n.name \n" +
            "end ) as conditionName,\n" +
            "labels(n)[0] as conditionType  \n" +
            "order by r.状态 desc, id(r) desc",
            countQuery = "match(m)-[r]->(n) \n" +
                    "where 1 = 1 \n" +
                    "and (m:实验室检查套餐名 or m:辅助检查名称 or m:辅助检查子项目名称 or m:医保手术和操作名称 or m:药品注册名称) \n" +
                    "and (n:性别 or n:实验室检查 or n:临床表现 or n:辅助检查名称描述 or n:疾病 or n:服用药品 or n:药物过敏原 or n:食物过敏原 or n:手术 or n:禁忌人群 or n:过敏原 or n:体征 or n:禁忌医疗器械及物品 or n:年龄 or n:给药途径 or n:开单项互斥) \n" +
                    //"and (case {labels} is not null and size({labels})>0 when true then labels(m)[0] in {labels} else 1 = 1 end ) \n" +
                    "and (case {conceptName} is not null and {conceptName} <> '' when true then m.name contains {conceptName} else 1 = 1 end ) \n" +
                    "and (case {conditionName} is not null and {conditionName} <> '' when true then " +
                    "(case  \n" +
                    "when exists(n.禁忌人群指标) then toLower(n.禁忌人群指标) contains toLower({conditionName}) \n" +
                    "when exists(n.实验室检查指标) then toLower(n.实验室检查指标) contains toLower({conditionName}) \n" +
                    "when exists(n.体征指标) then toLower(n.体征指标) contains toLower({conditionName}) \n" +
                    "else toLower(n.`name`) contains toLower({conditionName}) \n" +
                    "end ) \n" +
                    "else 1 = 1 " +
                    "end ) \n" +
                    "and (case {relationStatus} is not null  when true then r.状态 = {relationStatus} else 1 = 1 end ) \n" +
                    "and type(r) contains('禁忌') \n" +
                    "and m.状态=1 \n" +
                    "and n.状态=1 \n" +
                    "return count(n)")
    Page<BillItem> getPage(@Param("labels") List<String> labels,
                           @Param("conceptName") String conceptName,
                           @Param("conditionName") String conditionName,
                           @Param("relationStatus") Integer relationStatus,
                           Pageable pageable);

    @Query("match (m)-[r]->(n) " +
            "where type(r) contains '禁忌' " +
            "and labels(m)[0] = $conceptLabel " +
            "return distinct type(r)")
    List<String> getRelationTypes(@Param("conceptLabel") String conceptLabel);

    @Query("MATCH (n) \n" +
            "WHERE (toLower(n.`name`) = toLower($name) OR toLower(n.`拼音编码`) = toLower($pycode)) \n" +
            //"AND labels(n)[0] in {labels}  \n" +
            "and (n:性别 or n:实验室检查 or n:临床表现 or n:辅助检查名称描述 or n:疾病 or n:服用药品 or n:药物过敏原 or n:食物过敏原 or n:手术 or n:禁忌人群 or n:过敏原 or n:体征 or n:禁忌医疗器械及物品 or n:年龄 or n:给药途径 or n:开单项互斥) \n" +
            "AND not(exists(n.禁忌人群指标) or exists(n.实验室检查指标) or exists(n.体征指标)) \n" +
            "AND (case {notInIds} is not null and size({notInIds})>0 when true then not(id(n) in {notInIds}) else 1 = 1 end ) \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conditionLabel,n.最小值 as minValue,n.最大值 as maxValue,n.单位 as unit,n.范围 as range \n" +
            "union \n" +
            "MATCH (n) \n " +
            "WHERE (toLower(n.`name`) starts with toLower($name) OR toLower(n.`拼音编码`) starts with toLower($pycode)) \n" +
            //"AND labels(n)[0] in {labels}  \n" +
            "and (n:性别 or n:实验室检查 or n:临床表现 or n:辅助检查名称描述 or n:疾病 or n:服用药品 or n:药物过敏原 or n:食物过敏原 or n:手术 or n:禁忌人群 or n:过敏原 or n:体征 or n:禁忌医疗器械及物品 or n:年龄 or n:给药途径 or n:开单项互斥) \n" +
            "AND not(exists(n.禁忌人群指标) or exists(n.实验室检查指标) or exists(n.体征指标)) \n" +
            "AND (case {notInIds} is not null and size({notInIds})>0 when true then not(id(n) in {notInIds}) else 1 = 1 end ) \n" +
            "WITH n SKIP 0 LIMIT $size \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conditionLabel,n.最小值 as minValue,n.最大值 as maxValue,n.单位 as unit,n.范围 as range \n" +
            "union \n" +
            "MATCH (n) \n" +
            "WHERE (toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($pycode)) \n" +
            //"AND labels(n)[0] in {labels}  \n" +
            "and (n:性别 or n:实验室检查 or n:临床表现 or n:辅助检查名称描述 or n:疾病 or n:服用药品 or n:药物过敏原 or n:食物过敏原 or n:手术 or n:禁忌人群 or n:过敏原 or n:体征 or n:禁忌医疗器械及物品 or n:年龄 or n:给药途径 or n:开单项互斥) \n" +
            "AND not(exists(n.禁忌人群指标) or exists(n.实验室检查指标) or exists(n.体征指标)) \n" +
            "AND (case {notInIds} is not null and size({notInIds})>0 when true then not(id(n) in {notInIds}) else 1 = 1 end ) \n" +
            "WITH n SKIP 0 LIMIT $size \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conditionLabel,n.最小值 as minValue,n.最大值 as maxValue,n.单位 as unit,n.范围 as range \n")
    List<BillCondition> conditionIndex(@Param("labels") List<String> labels,
                                       @Param("notInIds") List<Long> notInIds,
                                       @Param("name") String name,
                                       @Param("pycode") String pycode,
                                       @Param("size") Integer size);

    @Query("MATCH(m)-[r]->(n) \n" +
            "where m.name={conceptName} \n" +
            "and {conceptLabel} in labels(m) \n" +
            "and type(r) contains '禁忌' \n" +
            "return ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conditionLabel \n")
    List<BillCondition> getNotInConditions(@Param("conceptName") String conceptName, @Param("conceptLabel") String conceptLabel);

    @Query("MATCH (n) \n" +
            "WHERE (toLower(n.`name`) = toLower($name) OR toLower(n.`拼音编码`) = toLower($pycode))  \n" +
            "AND labels(n)[0] in {labels}  \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conceptLabel \n" +
            "union \n" +
            "MATCH (n) \n " +
            "WHERE (toLower(n.`name`) starts with toLower($name) OR toLower(n.`拼音编码`) starts with toLower($pycode)) \n " +
            "AND labels(n)[0] in {labels}  \n" +
            "WITH n SKIP 0 LIMIT $size \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conceptLabel  \n" +
            "union \n" +
            "MATCH (n) \n" +
            "WHERE (toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($pycode))  \n" +
            "AND labels(n)[0] in {labels}  \n" +
            "WITH n SKIP 0 LIMIT $size \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conceptLabel \n")
    List<BillConcept> conceptIndex(@Param("labels") List<String> labels,
                                   @Param("name") String name,
                                   @Param("pycode") String pycode,
                                   @Param("size") Integer size);

    @Query("MATCH (m)-[r]->(n) \n" +
            "where m.name={conceptName} \n" +
            "and n.name={conditionName}\n" +
            "and type(r) ={relationType} \n" +
            "and {conceptLabel} in labels(m)\n" +
            "and {conditionLabel} in labels(n)\n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conditionLabel,n.最小值 as minValue,n.最大值 as maxValue,n.单位 as unit,n.范围 as range \n")
    BillCondition isExistCondition(@Param("conceptName") String conceptName,
                                   @Param("conceptLabel") String conceptLabel,
                                   @Param("relationType") String relationType,
                                   @Param("conditionName") String conditionName,
                                   @Param("conditionLabel") String conditionLabel);

    @Query("MATCH (m)-[r]->(n) \n" +
            "where id(r) ={relationId} \n" +
            "RETURN id(m) as conceptId,\n" +
            "m.name as conceptName,\n" +
            "labels(m)[0] as conceptLabel,\n" +
            "m.状态 as conceptStatus,\n" +
            "r.规则类型 as ruleType,\n" +
            "id(r) as relationId,\n" +
            "type(r) as relationName,\n" +
            "r.状态 as relationStatus,\n" +
            "id(n) as conditionId,\n" +
            //" n.name as conditionName," +
            "(case  \n" +
            "when exists(n.禁忌人群指标) then n.禁忌人群指标 \n" +
            "when exists(n.实验室检查指标) then n.实验室检查指标 \n" +
            "when exists(n.体征指标) then n.体征指标 \n" +
            "else n.name \n" +
            "end ) as conditionName,\n" +
            "labels(n)[0] as conditionLabel,\n" +
            "n.最小值 as conditionMinValue,\n" +
            "n.最大值 as conditionMaxValue,\n" +
            "n.单位 as conditionUnit,\n" +
            "n.范围 as conditionRange,\n" +
            "n.状态 as conditionStatus")
    BillItem getRecordByRelationId(@Param("relationId") Long relationId);

    @Query("MATCH (m)-[r]->(n) \n" +
            "where m.name ={conceptName} \n" +
            "and n.name={conditionName} \n" +
            "and type(r) ={relationType} \n" +
            "RETURN id(m) as conceptId,\n" +
            "m.name as conceptName,\n" +
            "labels(m)[0] as conceptLabel,\n" +
            "m.状态 as conceptStatus,\n" +
            "r.规则类型 as ruleType,\n" +
            "id(r) as relationId,\n" +
            "type(r) as relationName,\n" +
            "r.状态 as relationStatus,\n" +
            "id(n) as conditionId,\n" +
            "n.name as conditionName,\n" +
            "labels(n)[0] as conditionLabel,\n" +
            "n.最小值 as conditionMinValue,\n" +
            "n.最大值 as conditionMaxValue,\n" +
            "n.单位 as conditionUnit,\n" +
            "n.范围 as conditionRange,\n" +
            "n.状态 as conditionStatus")
    BillItem isExistRelation(@Param("conceptName") String conceptName,
                             @Param("conditionName") String conditionName,
                             @Param("relationType") String relationType);
}