package com.diagbot.repository;

import com.diagbot.entity.node.EntityBaseInfo;
import com.diagbot.entity.node.EntityInfo;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/12/14 13:26
 */
public interface EntityInfoRepository extends Neo4jRepository<EntityBaseInfo,Long> {
    @Query(value = "with {labels} as nlabels\n" +
            "match(n)\n" +
            "where any(label in labels(n) where label in nlabels)\n" +
            "and (case when {name} is not null and {name} <> '' " +
            "then " +
            "((toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($name))) " +
            "else 1 = 1 " +
            "end ) \n" +
            "and (case when {status} is not null " +
            "then " +
            "n.状态 = {status} " +
            "else 1 = 1 " +
            "end ) \n" +
            "and (case when {is_kl} is not null " +
            "then " +
            "n.静态知识标识 = {is_kl} " +
            "else 1 = 1 " +
            "end ) \n" +
            "and not(exists(n.禁忌人群指标)) \n" +
            "and not(exists(n.体征指标)) \n" +
            "and not(exists(n.实验室检查指标)) \n" +
            "return id(n) as id,n.name as name,n.拼音编码  as pycode,labels(n)[0] as labelType,n.状态 as status,n.静态知识标识 as is_kl\n" +
            "order by n.状态 desc, id(n) desc",
            countQuery = "with {labels} as nlabels\n" +
                    "match(n)\n" +
                    "where any(label in labels(n) where label in nlabels)\n" +
                    "and (case when {name} is not null and {name} <> '' " +
                    "then " +
                    "((toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($name))) " +
                    "else 1 = 1 " +
                    "end ) \n" +
                    "and (case when {status} is not null " +
                    "then " +
                    "n.状态 = {status} " +
                    "else 1 = 1 " +
                    "end ) \n" +
                    "and (case when {is_kl} is not null " +
                    "then " +
                    "n.静态知识标识 = {is_kl} " +
                    "else 1 = 1 " +
                    "end ) \n" +
                    "and not(exists(n.禁忌人群指标)) \n" +
                    "and not(exists(n.体征指标)) \n" +
                    "and not(exists(n.实验室检查指标)) \n" +
                    "return count(*)")
    Page<EntityInfo> getEntityPage(@Param("labels") List<String> labels,
                                   @Param("name") String name,
                                   @Param("status") Integer status,
                                   @Param("is_kl") Integer is_kl,
                                   Pageable pageable);

    @Query(value = "match(n) where n.name={name} and {label} in labels(n) \n " +
            "return id(n) as id,n.name as name,n.拼音编码  as pycode,labels(n)[0] as labelType,n.状态 as status,n.静态知识标识 as is_kl\n")
    List<EntityInfo> existNodes(@Param("name") String name,
                                @Param("label") String label);

    @Query("merge (n{id:{name}})  " +
            "on create set  " +
            "n.拼音编码={pycode}," +
            "n.静态知识标识={is_kl}," +
            "n.状态={status}," +
            "n.最大值={maxValue}," +
            "n.最小值={minValue}," +
            "n.范围={range}," +
            "n.单位={unit}, " +
            "n.实验室检查指标={indication} " +
            "on match set  " +
            "n.拼音编码={pycode}," +
            "n.静态知识标识={is_kl}," +
            "n.状态={status}," +
            "n.最大值={maxValue}," +
            "n.最小值={minValue}," +
            "n.范围={range}," +
            "n.单位={unit}, " +
            "n.实验室检查指标={indication} " +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    Boolean updateEntity(@Param("id") Long id);

    @Query(value = "match(n)\n" +
            "where id(n)={id}\n" +
            "return id(n) as id,\n" +
            "n.name as name,\n" +
            "n.拼音编码 as pycode,\n" +
            "n.状态 as status,\n" +
            "n.静态知识标识 as is_kl,\n" +
            "labels(n)[0] as labelType,\n" +
            "n.年龄 as age,\n" +
            "n.age as ageRange,\n" +
            "n.性别 as gender,\n" +
            "n.单位 as unit,\n" +
            "case labels(n)[0] \n" +
            "when \"医保疾病名称\" then n.疾病别称\n" +
            "when \"药物过敏原\" then n.名称\n" +
            "when \"药品注册名称\" then n.名称\n" +
            "end  as alias,\n" +
            "n.结果 as result,\n" +
            "n.最大值 as maxval,\n" +
            "n.最小值 as minval,\n" +
            "n.范围 as range,\n" +
            "n.分类 as category,\n" +
            "n.dept as dept,\n" +
            //"n.年龄max as maxAge,\n" +
            //"n.年龄min as minAge,\n" +
            "n.手术级别 as opgrade,\n" +
            "n.高危条件 as highriskcond,\n" +
            "n.special as special,\n" +
            "n.药品类型 as medtype,\n" +
            "n.药品高危级别 as risklevel,\n" +
            "n.禁忌给药途径 as conflict_geiyao,\n" +
            "n.禁忌性别 as conflict_gender," +
            "n.最小包装数量 as min_pack_num," +
            "n.注册剂型 as reg_JiXin," +
            "n.药品本位码 as benWei_Code," +
            "n.最小包装单位 as min_pack_unit," +
            "n.注册规格 as reg_GuiGe," +
            "n.包装材质 as pack_CaiLiao," +
            "n.药品企业 as producer," +
            "n.批准文号 as license_num," +
            "n.最小制剂单位 as min_prod_unit," +
            "n.标准注册剂型 as stand_reg_JiXin")
    EntityInfo getById(@Param("id") Long id);


    @Query("match (n) \n" +
            "where id(n) = :#{#entityInfo.id} \n" +
            "set  " +
            "n.name = :#{#entityInfo.name}," +
            "n.拼音编码 = :#{#entityInfo.pycode}," +
            "n.静态知识标识 = :#{#entityInfo.is_kl}," +
            "n.状态 = :#{#entityInfo.status}")
    void update(@Param("entityInfo") EntityInfo entityInfo);

    @Query("create (n) \n" +
            "set  " +
            "labels(n)=[:#{#entityInfo.labelType}]," +
            "n.name = :#{#entityInfo.name}," +
            "n.拼音编码 = :#{#entityInfo.pycode}," +
            "n.静态知识标识 = :#{#entityInfo.is_kl}," +
            "n.状态 = :#{#entityInfo.status}" +
            "with id(n) as nodeId\n" +
            "match (n)\n" +
            "where id(n) = nodeId\n" +
            "set labels(n)=[:#{#entityInfo.labelType}]")
    void create(@Param("entityInfo") EntityInfo entityInfo);
}