package com.diagbot.repository;

import com.diagbot.entity.node.EntityInfo;
import com.diagbot.entity.node.TransfusionRemind;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;


public interface TransfusionRemindRepository extends Neo4jRepository<TransfusionRemind, Long> {

    List<TransfusionRemind> findByNameIs(String name);

    List<TransfusionRemind> findByNameContaining(String name);

    @Query("MATCH (n:输血提醒指标) where n.输注类型 contains {name} return distinct(n.输注类型)")
    List<String> findTransfusionByName(@Param("name") String name);

    @Query("MATCH (n:输血提醒指标) where n.输注类型 contains {name} return n")
    List<TransfusionRemind> findByTransfusionName(@Param("name") String name);

    @Query("MATCH (n:输血提醒指标) where n.输注类型 in {names} return distinct(n.输注类型)")
    List<String> findTransfusionByTypeIn(@Param("names") List<String> names);

    @Query("MATCH (n:输血提醒指标) return distinct(n.输注类型) as type LIMIT $(size)")
    List<String> getTransfusionNames(@Param("size") Integer size);

    @Query("create (n:输血提醒指标) \n" +
            "set  " +
            "n.name = :#{#entityInfo.name}," +
            "n.拼音编码 = :#{#entityInfo.pycode}," +
            "n.静态知识标识 = :#{#entityInfo.is_kl}," +
            "n.状态 = :#{#entityInfo.status}" +
            "\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    TransfusionRemind create(@Param("entityInfo") EntityInfo entityInfo);
}
