package com.diagbot.repository;

import com.diagbot.entity.node.EntityInfo;
import com.diagbot.entity.node.Vital;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;


public interface VitalRepository extends Neo4jRepository<Vital, Long> {

    List<Vital> findByNameIs(String name);

    List<Vital> findByNameIn(List<String> names);

    List<Vital> findByNameContaining(String name);

    @Query("MATCH (n:`体征`) WHERE toLower(n.`name`) = toLower($name) \n" +
            "OR toLower(n.`拼音编码`) = toLower($pycode)  \n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`体征`) WHERE toLower(n.`name`) starts with toLower($name) \n" +
            "OR toLower(n.`拼音编码`) starts with toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`体征`) WHERE toLower(n.`name`) CONTAINS toLower($name) \n" +
            "OR toLower(n.`拼音编码`) CONTAINS toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    List<Vital> index(@Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);


    @Query("MATCH (n:体征) return distinct(n.name) as name LIMIT {size}")
    List<String> getVitalNames(@Param("size") Integer size);

    @Query("merge (n:体征{name:{name}})  " +
            "on create set  " +
            "n.拼音编码={pycode}," +
            "n.静态知识标识={is_kl}," +
            "n.状态={status}," +
            "n.最大值={maxValue}," +
            "n.最小值={minValue}," +
            "n.范围={range}," +
            "n.单位={unit}, " +
            "n.体征指标={indication} " +
            "on match set  " +
            "n.拼音编码={pycode}," +
            "n.静态知识标识={is_kl}," +
            "n.状态={status}," +
            "n.最大值={maxValue}," +
            "n.最小值={minValue}," +
            "n.范围={range}," +
            "n.单位={unit}, " +
            "n.体征指标={indication} " +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    Vital createOrUpdate(@Param("name") String name,
                         @Param("is_kl") Integer is_kl,
                         @Param("pycode") String pycode,
                         @Param("status") Integer status,
                         @Param("maxValue") Double maxValue,
                         @Param("minValue") Double minValue,
                         @Param("range") Integer range,
                         @Param("unit") String unit,
                         @Param("indication") String indication);

    @Query("match (n:体征) \n" +
            "where id(n) = :#{#entityInfo.id} \n" +
            "set  " +
            "n.name = :#{#entityInfo.name}," +
            "n.拼音编码 = :#{#entityInfo.pycode}," +
            "n.静态知识标识 = :#{#entityInfo.is_kl}," +
            "n.状态 = :#{#entityInfo.status}," +
            "n.单位 = {unit}" +
            "\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    Vital update(@Param("entityInfo") EntityInfo entityInfo,
                 @Param("unit") String unit);

    @Query("create (n:体征) \n" +
            "set  " +
            "n.name = :#{#entityInfo.name}," +
            "n.拼音编码 = :#{#entityInfo.pycode}," +
            "n.静态知识标识 = :#{#entityInfo.is_kl}," +
            "n.状态 = :#{#entityInfo.status}," +
            "n.单位 = {unit}" +
            "\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    Vital create(@Param("entityInfo") EntityInfo entityInfo,
                 @Param("unit") String unit);
}

